(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(**************************************************************************)
(* variable declarations                                                  *)
(**************************************************************************)

open Types

type declarationkind =
    Logical
  | Mutable      (* only used in roles and scenarios *)
  | ProcessKind  (* only used in formulas *)

val string_of_declarationkind: declarationkind -> string

(* normal variable declaration: we associate to a name   *)
(* - the kind of the variable (logical or mutable)       *)
(* - its type                                            *)
type declaration = 
    string * (declarationkind * type_expression)

val string_of_declaration: declaration -> string

(* (rename_expr alpha dlist) yields a copy of dlist where every   *)
(* variable x defined in the association list alpha has been      *) 
(* replaced by alpha(x).                                          *)
val rename_dlist: (string * string) list -> declaration list ->
  declaration list


(* string_of_parlist converts a declaration list to a string in the *)
(* style of a parameterlist, that is all on one single line, and    *)
(* separated by ";"                                                 *)
val string_of_parlist: declaration list -> string

(* declaration of a variable with explicit skolemisation      *)
(* we associate to a name                                     *)
(* - the list of types of the variables from which it depends *)
(* - the kind of the variable (logical or mutable)            *)
(* - its type                                                 *)

type sk_declaration =
    string * ( type_expression list * declarationkind * type_expression)

val string_of_sk_declaration: sk_declaration -> string

(* flatten a declaration of a list of variables into a list of *)
(* proper declarations.                                        *)
val flatten_declaration :
  string list ->
  (declarationkind * type_expression) ->
  declaration list
val flatten_sk_declaration:
  (string * (type_expression list)) list ->
  (declarationkind * type_expression) ->
  sk_declaration list

(* unskolemize strips the skolem part from a declaration list *)
val unskolemize: sk_declaration list -> declaration list

(* make_dec_logical constructs a declaration list from a list of    *)
(* pairs (variable_name, type_expression) by considering them all   *)
(* as declarations of logical variables.                            *)
val make_dec_logical: (string * type_expression) list -> declaration list

(* check that a declaration list is well-formed:              *)
(* - no variable declared twice                               *)
(* - all type expressions variable-free                       *)
(* - all type expressions arrow-free                          *)
exception Exc_declarations of string
val check_declarations :  declaration list -> unit


