(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

exception Exc_prouve of string


(************************************************************)
(*           Abstract syntax of PROUVE files                *)
(************************************************************)

module AbstractSyntax :
sig
  (* type expressions *)
  type type_expression =
      Typeconst  (* a primitive type *)
	of string
    | Typevar    (* a type variable as used in polymorphic types *) 
	of string
    | Typeapply  (* application of a type constructor, e.g. list *)
	of string * type_expression list
    | Typearrow  (* a function type *)
	of type_expression * type_expression
    | Typetuple  (* a product type *)
	of type_expression list
    | Typeundef  (* an anonymous type variable, used e.g. to type *)
        (* an empty list.                                *)
  val string_of_type_expression: type_expression -> string
    (* test whether a type is simple, that is only built of type constants, *)
    (* tupling, and arrows.                                                 *)
  val is_simple_type: type_expression -> bool

    
  (* a  signature declaration, assigning to names of symbols  *)
  (* their type.                                              *)
  type simple_signature = (string * type_expression ) list
  type signature = simple_signature * simple_signature
  val print_signature: out_channel -> signature -> unit
  val default_signature : (string * type_expression) list
    
  (* the two different kinds of declarations *)
  type declarationkind =
      Logical     (* variable that can be asigned only once *)
    | Mutable     (* variable that can be assigned any number of times *)
                  (* this is omly used in roles and scenarios *)
    | ProcessKind (* variable denoting a process. *)
                  (* this is only used in the logic. *)
  val string_of_declarationkind: declarationkind -> string
    
  (* variable declaration: we associate to a name          *)
  (* - the kind of the variable (logical or mutable)       *)
  (* - its type                                            *)
  type declaration = 
      string * (declarationkind * type_expression)
	
  (* expressions *)
  type expression =
      Ident   (* a variable or a constant *)
	of string
    | Int     (* an integer value *)
	of int
    | Apply   (* application of a function to expressions *)
	of string * expression list
    | Infix   (* application of a mixfix operator to two expressions *)
	of string * expression * expression
    | Inverse (* application of the Inverse operator *)
	of expression
    | Tuple   (* a tuple of expressions *)
	of expression list
    | Liste   (* a list of expressions *)
	of expression list
    | AssocList  (* an associaton list *)
	of (expression*expression) list
  val string_of_expression: expression -> string
    
  (* specification of axioms. This part consists of              *)
  (* - the declaration of the variables to be used in the axioms *)
  (* - the equational axioms (which are just a special case of   *)
  (*   expressions.                                              *)
  type axioms = declaration list * expression list
      
      
  (* patterns are used in several kinds of instructions. A primitive *)
  (* pattern is an expression, and a pattern is a list of primitive  *)
  (* patterns representing their disjunction.                        *)
      
  type pattern = expression list
  val string_of_pattern: pattern -> string
    
    
  (* instructions as used in the definition of roles *)
  type instruction =
      IBlock (* a block instruction *)
	of string * block
    | ISend  (* a send instrcution *)
	of string * expression
    | IRecv  (* a receive instruction, using pattern matching on the *)
        (* received message.                                    *)
	of string * pattern
    | IFail  of string (* the failing instruction *)
    | IAssign (* assignment of the value of an expression to a variable *) 
	of string * string * expression
    | INew   (* assign a variable a new value *)
	of string * string
    | IMatch (* match the value of an expression against a pattern  *)
        (* thus binding the variables of the pattern.          *)
	of string * expression * pattern
    | IConditional (* conditional instruction *)
	of string * expression * instruction list * instruction list
    | IChoice of string * (instruction list) list (* nondeterministic choice *)
    | ICase  (* case instruction. Every clause consists of a pattern *)
        (* and an instruction list to be executed in case of    *)
        (* matching. The final istruction list is executed in   *)
        (* case that none of the patterns has matched.          *)
	of string *
	  expression * (pattern * instruction list) list * instruction list
  and block =
      declaration list * instruction list
	
  (* a role definition consists of          *)
  (* - its name                             *)
  (* - its parameter list                   *)
  (* - its body, which is a block           *)
  type role = 
      Role of string * declaration list * block
  val print_role: out_channel -> role -> unit
    (* (get_form_parlist name rolelist) returns the list of formal *)
    (* parameters of role name in the definitions rolelist.        *)
  val get_form_parlist : string -> role list -> string list    
    
  (* declaration of a variable with explicit skolemisation      *)
  (* we associate to a name                                     *)
  (* - the list of types of the variables from which it depends *)
  (* - the kind of the variable (logical or mutable)            *)
  (* - its type                                                 *)
  type sk_declaration =
      string * ( type_expression list * declarationkind * type_expression)
	
	
  (* the type of integers extended by omega (= infinite) *)
  type omint = OmInt of int | Omega
      
      
  (* the type of integer intervals *)
  type interval = omint * omint
      
      
  (* Scenarios. Note that scenarios may use variables with explicit *)
  (* skolemisation. We hence have to allow arbitrary expressions    *)
  (* at places where, in role definitions, we only allowed simple   *)
      
  type scenario =
      SCall (* invocation of a role *)
	of string * string * expression list
    | SNew (* creation of a new value *)
	of string * expression
    | SForever (* execute the given scenario list sequentially and  *)
        (* in an infinite sequential loop.                   *)   
	of string * scenario list
    | SParallel(* execute all the scenarios in parallel *) 
	of string * scenario list
    | SSequence (* execute all the scenarios sequentially *)
	of string * scenario list
    | SAssign  (* assignment of the value of the 2nd expression  *)
        (* to the memory location designated by the 1st   *)
        (* expression.                                    *)
	of string * expression * expression
    | SConditional (* conditional instruction *)
	of string * expression * scenario list * scenario list
    | SForall (* execute the scenario for all possible values of  *)
        (* the variables in the declaration list, but       *)
        (* restricted to soem relation satisfying the       *)
        (* constraint.                                      *)
	of string * declaration list * interval list * scenario
    | SExists (* execute the scenario for some tuple of values     *)
        (* of the variables declared in the declaration list *)  
	of string * declaration list * scenario
  val print_scenario: out_channel -> scenario -> unit
    
  type spec = signature * axioms * role list * sk_declaration list * scenario
  val print_spec: out_channel -> spec -> unit


  (* (parse_spec fname) returns the abstract syntax of the PROUV    *)
  (* specification in the file fname. Raises an exception Exc_prouve *)
  (* with a string indicating fname and the nature of the            *)
  (* exception.                                                      *)
  val parse_spec: string -> spec 
    
end

(*****************************************************************)
(* Module for Logical Formulas and Assertions                    *)
(*****************************************************************)

module Logic :
sig
(* an assertion consists of a list of public symbols, an initial condition *)
(* and a formula which is suppose to always hold.                          *)

  type formula =
      Ftrue
    | Fatom of AbstractSyntax.expression
    | Fsecret of AbstractSyntax.expression
    | Fprocessat of string * string
    | Fscenarioat of string
    | Fand of formula * formula
    | For of formula * formula
    | Fimplies of formula * formula
    | Fnot of formula
    | Fforall of AbstractSyntax.declaration list * formula
    | Fexists of AbstractSyntax.declaration list * formula
    | Funique of AbstractSyntax.declaration list * formula
    | Fbox of formula

  type assertion =
      string * string list * formula * formula
	
  (* (parse_assertion fname) returns the abstract syntax of the PROUV *)
  (* specificationin the file fname. Raises an exception Exc_prouve    *)
  (* with a string indicating fname and the nature of the exception.   *)
  val parse_assertion: string -> assertion 
	
end

(*****************************************************************)
(* Module for handling Term Rewriting Systems                    *)
(*****************************************************************)

module TermRewriting :
sig
  (* the type of term rewriting systems *)
  type trs = (AbstractSyntax.expression * AbstractSyntax.expression ) list

  (* string representation of a trs *)
  val trs_to_string : trs -> string

  (* the type of substitutions *)
  type substitution = (string * AbstractSyntax.expression) list

  (* apply a substitution to a term *)
  val apply_subst : substitution -> AbstractSyntax.expression ->
    AbstractSyntax.expression
end


(************************************************************************)
(* Miscellaneous                                                        *)
(************************************************************************)

module Misc :
sig
  val remove_double_occurrences: 'a list -> 'a list
end
