(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUV language                 *)
(*                                                                        *)
(*  Copyright (C) 2004, 2005 Ralf Treinen                                 *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@lsv.ens-cachan.fr>                              *)
(*  Laboratoire Spcification et Vrification                             *)
(*  cole Normale Suprieure de Cachan, CNRS UMR 8643                     *)
(*  61, avenue du Prsident Wilson                                        *)
(*  94235 CACHAN Cedex - France                                           *)
(**************************************************************************)

(*********************************************)
(*            Role Definitions               *)
(*********************************************)

open Types
open Expressions
open Declarations

(* raw abstract syntax *)
type rawinstruction =
    IRawLabeled of string * rawinstruction
  | IRawBlock of rawblock
  | IRawSend of expression
  | IRawRecv of pattern
  | IRawFail
  | IRawAssign of string * expression
  | IRawNew of string
  | IRawMatch of expression * pattern
  | IRawConditional of expression * rawinstruction list * rawinstruction list
  | IRawChoice of rawinstruction list list
  | IRawCase of expression * (pattern * rawinstruction list) list
      * rawinstruction list
and rawblock =
    declaration list * rawinstruction list

(* processed abstract sysntax *)
type instruction =
    IBlock of string*block
  | ISend of string*expression
  | IRecv of string*pattern
  | IFail of string
  | IAssign of string * string * expression
  | INew of string * string
  | IMatch of string * expression * pattern
  | IConditional of string * expression * instruction list * instruction list
  | IChoice of string * (instruction list) list
  | ICase of string *
      expression * (pattern * instruction list) list * instruction list
and block =
    declaration list * instruction list

(* process a block of instructions. The string argument is the prefix *)
(* used in the generation of default labels for instructions.         *)
val process_instblock: string -> rawblock -> block

(* a role definition consists of          *)
(* - its name                             *)
(* - its parameter list                   *)
(* - its body, which is a block           *)
type role = 
    Role of string * declaration list * block

(* (print_role oc r) prints role r to the output channel oc *)
val print_role: out_channel -> role -> unit

(* (rename_expr alpha ilist) yields a copy of ilist where every     *)
(* variable x defined in the association list alpha has been        *) 
(* replaced by alpha(x).                                            *)
val rename_ilist: (string * string) list -> instruction list ->
  instruction list

(* get a sorted list of all integer constants used in a role *)
val int_constants : role -> int list

(* (bound_vars_ilist il) returns the list of the names of bound variables *)
(* declared somewhere in il. This list might contain multiple occurrences *)
(* of the same name.                                                      *)
val bound_vars_ilist: instruction list -> string list

(* check the typing of a list of roles                       *)
exception Exc_roles of string
val check_types_roles: Signatures.simple_signature -> role list -> unit



(**********  extraction information about roles *************************)

(* (gather_parameter_declarations rolelist) returns an association list *)
(* which associates to any role in rolelist the declaration list of its *)
(* formal parameters.                                                   *)
val gather_parameter_declarations:
  role list -> (string * (declaration list)) list

(* (gather_toplevel_declarations rolelist) returns an association list  *)
(* which associates to any role in rolelist the declaration list of its *)
(* toplevel identifiers (formal parameters or variables defined at the  *)
(* topmost level of the body).                                          *)
val gather_toplevel_declarations:
  role list -> (string * (declaration list)) list

(* (get_form_parlist name rolelist) returns the list of formal parameters *)
(* of role name in the definitions rolelist.                              *)
val get_form_parlist : string -> role list -> string list

(* (gather_checkpoints rolelist) returns an association list that *)
(* associates to any role in rolelist the list of its checkpoints *)
val gather_checkpoints : role list -> (string * string list) list






